@@ -452,7 +452,7 @@ class TestApprox:
         expected = "4.0e-06"
         result = testdir.runpytest()
         result.stdout.fnmatch_lines(
-            ["*At index 0 diff: 3 != 4 * {}".format(expected), "=* 1 failed in *="]
+            ["*At index 0 diff: 3 != 4 ± {}".format(expected), "=* 1 failed in *="]
         )
 
     @pytest.mark.parametrize(
