@@ -10,16 +10,14 @@ import py
 
 import _pytest._code
 import pytest
-from _pytest import main
 from _pytest import outcomes
 from _pytest import reports
 from _pytest import runner
-from _pytest.outcomes import Exit
-from _pytest.outcomes import Failed
+from _pytest.compat import TYPE_CHECKING
+from _pytest.config import ExitCode
 from _pytest.outcomes import OutcomeException
-from _pytest.outcomes import Skipped
 
-if False:  # TYPE_CHECKING
+if TYPE_CHECKING:
     from typing import Type
 
 
@@ -384,7 +382,7 @@ class BaseFunctionalTests:
             """
             )
         except SystemExit:
-            pytest.fail("runner did not catch SystemExit")
+            assert False, "runner did not catch SystemExit"
         rep = reports[1]
         assert rep.failed
         assert rep.when == "call"
@@ -398,10 +396,10 @@ class BaseFunctionalTests:
                     raise pytest.exit.Exception()
             """
             )
-        except Exit:
+        except pytest.exit.Exception:
             pass
         else:
-            pytest.fail("did not raise")
+            assert False, "did not raise"
 
 
 class TestExecutionNonForked(BaseFunctionalTests):
@@ -422,7 +420,7 @@ class TestExecutionNonForked(BaseFunctionalTests):
         except KeyboardInterrupt:
             pass
         else:
-            pytest.fail("did not raise")
+            assert False, "did not raise"
 
 
 class TestExecutionForked(BaseFunctionalTests):
@@ -557,19 +555,17 @@ def test_outcomeexception_passes_except_Exception() -> None:
         try:
             raise outcomes.OutcomeException("test")
         except Exception:
-            pass
+            raise NotImplementedError()
 
 
 def test_pytest_exit() -> None:
-    assert Exit == pytest.exit.Exception  # type: ignore
-    with pytest.raises(Exit) as excinfo:
+    with pytest.raises(pytest.exit.Exception) as excinfo:
         pytest.exit("hello")
-    assert excinfo.errisinstance(Exit)
+    assert excinfo.errisinstance(pytest.exit.Exception)
 
 
 def test_pytest_fail() -> None:
-    assert Failed == pytest.fail.Exception  # type: ignore
-    with pytest.raises(Failed) as excinfo:
+    with pytest.raises(pytest.fail.Exception) as excinfo:
         pytest.fail("hello")
     s = excinfo.exconly(tryshort=True)
     assert s.startswith("Failed")
@@ -681,7 +677,7 @@ def test_pytest_fail_notrace_non_ascii(testdir) -> None:
 def test_pytest_no_tests_collected_exit_status(testdir) -> None:
     result = testdir.runpytest()
     result.stdout.fnmatch_lines(["*collected 0 items*"])
-    assert result.ret == main.ExitCode.NO_TESTS_COLLECTED
+    assert result.ret == ExitCode.NO_TESTS_COLLECTED
 
     testdir.makepyfile(
         test_foo="""
@@ -692,19 +688,19 @@ def test_pytest_no_tests_collected_exit_status(testdir) -> None:
     result = testdir.runpytest()
     result.stdout.fnmatch_lines(["*collected 1 item*"])
     result.stdout.fnmatch_lines(["*1 passed*"])
-    assert result.ret == main.ExitCode.OK
+    assert result.ret == ExitCode.OK
 
     result = testdir.runpytest("-k nonmatch")
     result.stdout.fnmatch_lines(["*collected 1 item*"])
     result.stdout.fnmatch_lines(["*1 deselected*"])
-    assert result.ret == main.ExitCode.NO_TESTS_COLLECTED
+    assert result.ret == ExitCode.NO_TESTS_COLLECTED
 
 
 def test_exception_printing_skip() -> None:
-    assert Skipped == pytest.skip.Exception  # type: ignore
+    assert pytest.skip.Exception == pytest.skip.Exception
     try:
         pytest.skip("hello")
-    except Skipped:
+    except pytest.skip.Exception:
         excinfo = _pytest._code.ExceptionInfo.from_current()
         s = excinfo.exconly(tryshort=True)
         assert s.startswith("Skipped")
@@ -721,7 +717,7 @@ def test_importorskip(monkeypatch) -> None:
         assert sysmod is sys
         # path = pytest.importorskip("os.path")
         # assert path == os.path
-        excinfo = pytest.raises(Skipped, f)
+        excinfo = pytest.raises(pytest.skip.Exception, f)
         assert excinfo is not None
         excrepr = excinfo.getrepr()
         assert excrepr is not None
@@ -735,13 +731,14 @@ def test_importorskip(monkeypatch) -> None:
         mod = types.ModuleType("hello123")
         mod.__version__ = "1.3"  # type: ignore
         monkeypatch.setitem(sys.modules, "hello123", mod)
-        with pytest.raises(Skipped):
+        with pytest.raises(pytest.skip.Exception):
             pytest.importorskip("hello123", minversion="1.3.1")
         mod2 = pytest.importorskip("hello123", minversion="1.3")
         assert mod2 == mod
-    except Skipped:
-        print(_pytest._code.ExceptionInfo.from_current())
-        pytest.fail("spurious skip")
+    except pytest.skip.Exception:  # pragma: no cover
+        assert False, "spurious skip: {}".format(
+            _pytest._code.ExceptionInfo.from_current()
+        )
 
 
 def test_importorskip_imports_last_module_part() -> None:
@@ -756,11 +753,12 @@ def test_importorskip_dev_module(monkeypatch) -> None:
         monkeypatch.setitem(sys.modules, "mockmodule", mod)
         mod2 = pytest.importorskip("mockmodule", minversion="0.12.0")
         assert mod2 == mod
-        with pytest.raises(Skipped):
+        with pytest.raises(pytest.skip.Exception):
             pytest.importorskip("mockmodule1", minversion="0.14.0")
-    except Skipped:
-        print(_pytest._code.ExceptionInfo.from_current())
-        pytest.fail("spurious skip")
+    except pytest.skip.Exception:  # pragma: no cover
+        assert False, "spurious skip: {}".format(
+            _pytest._code.ExceptionInfo.from_current()
+        )
 
 
 def test_importorskip_module_level(testdir) -> None:
@@ -1030,7 +1028,7 @@ def test_outcome_exception_bad_msg() -> None:
     """Check that OutcomeExceptions validate their input to prevent confusing errors (#5578)"""
 
     def func() -> None:
-        pass
+        raise NotImplementedError()
 
     expected = (
         "OutcomeException expected string as 'msg' parameter, got 'function' instead.\n"
